@@ -3,6 +3,7 @@ package oviewer
 import (
 	"bytes"
 	"context"
+	"path/filepath"
 	"reflect"
 	"testing"
 )
@@ -19,12 +20,12 @@ func TestOpenDocument(t *testing.T) {
 	}{
 		{
 			name:    "no file",
-			args:    args{fileName: "../testdata/nofile"},
+			args:    args{fileName: filepath.Join(testdata, "nofile")},
 			wantErr: true,
 		},
 		{
 			name:    "normal.txt",
-			args:    args{fileName: "../testdata/normal.txt"},
+			args:    args{fileName: filepath.Join(testdata, "normal.txt")},
 			wantErr: false,
 		},
 	}
@@ -82,9 +83,10 @@ func TestDocument_lineToContents(t *testing.T) {
 			if err := m.ReadAll(bytes.NewBufferString(tt.str)); err != nil {
 				t.Fatal(err)
 			}
-			<-m.eofCh
+			for !m.BufEOF() {
+			}
 			t.Logf("num:%d", m.BufEndNum())
-			got, err := m.contentsLN(tt.args.lN, tt.args.tabWidth)
+			got, err := m.contents(tt.args.lN, tt.args.tabWidth)
 			if (err != nil) != tt.wantErr {
 				t.Errorf("Document.lineToContents() error = %v, wantErr %v", err, tt.wantErr)
 				return
@@ -136,7 +138,8 @@ func TestDocument_Export(t *testing.T) {
 				t.Fatal(err)
 			}
 			w := &bytes.Buffer{}
-			<-m.eofCh
+			for !m.BufEOF() {
+			}
 			m.bottomLN = m.BufEndNum()
 			m.Export(w, tt.args.start, tt.args.end)
 			if gotW := w.String(); gotW != tt.wantW {
@@ -146,56 +149,6 @@ func TestDocument_Export(t *testing.T) {
 	}
 }
 
-func TestDocument_getContents(t *testing.T) {
-	type fields struct {
-		FileName string
-	}
-	type args struct {
-		lN       int
-		tabWidth int
-	}
-	tests := []struct {
-		name   string
-		fields fields
-		args   args
-		want   contents
-		want1  bool
-	}{
-		{
-			name: "test normal",
-			fields: fields{
-				FileName: "../testdata/normal.txt",
-			},
-			args: args{
-				lN:       0,
-				tabWidth: 8,
-			},
-			want:  parseString("khaki	mediumseagreen	steelblue	forestgreen	royalblue	mediumseagreen", 8),
-			want1: true,
-		},
-	}
-	for _, tt := range tests {
-		t.Run(tt.name, func(t *testing.T) {
-			m, err := OpenDocument(tt.fields.FileName)
-			if err != nil {
-				t.Fatalf("OpenDocument %s", err)
-			}
-
-			for !m.BufEOF() {
-			}
-			got, ok := m.getContents(tt.args.lN, tt.args.tabWidth)
-			if !reflect.DeepEqual(got, tt.want) {
-				g, _ := ContentsToStr(got)
-				w, _ := ContentsToStr(tt.want)
-				t.Errorf("Document.getContents() = [%v], want [%v]", g, w)
-			}
-			if ok != tt.want1 {
-				t.Errorf("Document.getContents() ok = %v, want %v", ok, tt.want1)
-			}
-		})
-	}
-}
-
 func TestDocument_searchLine(t *testing.T) {
 	type fields struct {
 		FileName string
@@ -215,7 +168,7 @@ func TestDocument_searchLine(t *testing.T) {
 		{
 			name: "test forward search",
 			fields: fields{
-				FileName: "../testdata/normal.txt",
+				FileName: filepath.Join(testdata, "normal.txt"),
 			},
 			args: args{
 				ctx:      context.Background(),
@@ -228,7 +181,7 @@ func TestDocument_searchLine(t *testing.T) {
 		{
 			name: "test forward search not found",
 			fields: fields{
-				FileName: "../testdata/normal.txt",
+				FileName: filepath.Join(testdata, "normal.txt"),
 			},
 			args: args{
 				ctx:      context.Background(),
@@ -280,7 +233,7 @@ func TestDocument_backSearchLine(t *testing.T) {
 		{
 			name: "test backword search",
 			fields: fields{
-				FileName: "../testdata/normal.txt",
+				FileName: filepath.Join(testdata, "normal.txt"),
 			},
 			args: args{
 				ctx:      context.Background(),
@@ -293,7 +246,7 @@ func TestDocument_backSearchLine(t *testing.T) {
 		{
 			name: "test backword search not found",
 			fields: fields{
-				FileName: "../testdata/normal.txt",
+				FileName: filepath.Join(testdata, "normal.txt"),
 			},
 			args: args{
 				ctx:      context.Background(),
