@@ -16,6 +16,10 @@ import (
 	"golang.org/x/term"
 )
 
+// bufSize is the size of the buffer used when reading the file.
+// This bufSize is used when only counting.
+const bufSize = 4096
+
 // FormFeed is the delimiter that separates the sections.
 // The default delimiter that separates single output from watch.
 const FormFeed = "\f"
@@ -23,6 +27,7 @@ const FormFeed = "\f"
 // firstRead first reads the file.
 // Fill the contents of the read file into the first chunk.
 func (m *Document) firstRead(reader *bufio.Reader) (*bufio.Reader, error) {
+	atomic.StoreInt32(&m.noNewlineEOF, 0)
 	chunk := m.chunks[0]
 	if err := m.fillChunk(chunk, reader, 0, true); err != nil {
 		if !errors.Is(err, io.EOF) {
@@ -49,10 +54,6 @@ func (m *Document) continueRead(reader *bufio.Reader) (*bufio.Reader, error) {
 		} else {
 			reader.Reset(m.file)
 		}
-	} else {
-		if chunkNum := m.lastChunkNum(); chunkNum != 0 {
-			m.loadedChunks.PeekOrAdd(chunkNum, struct{}{})
-		}
 	}
 	chunk := m.chunkForAdd()
 	start := len(chunk.lines)
@@ -77,15 +78,21 @@ func (m *Document) followRead(reader *bufio.Reader) (*bufio.Reader, error) {
 	if !m.FollowMode && !m.FollowAll {
 		return reader, nil
 	}
+
+	atomic.StoreInt32(&m.eof, 0)
+	chunk := m.chunks[m.lastChunkNum()]
+	start := len(chunk.lines) - 1
+	if atomic.LoadInt32(&m.noNewlineEOF) == 0 {
+		chunk := m.chunkForAdd()
+		start = len(chunk.lines)
+	}
+
 	if m.seekable {
 		if _, err := m.file.Seek(m.offset, io.SeekStart); err != nil {
 			return nil, fmt.Errorf("seek: %w", err)
 		}
 		reader = bufio.NewReader(m.file)
 	}
-
-	chunk := m.chunkForAdd()
-	start := len(chunk.lines)
 	if err := m.fillChunk(chunk, reader, start, true); err != nil {
 		if !errors.Is(err, io.EOF) {
 			return nil, err
@@ -107,7 +114,8 @@ func (m *Document) searchRead(reader *bufio.Reader, chunkNum int, searcher Searc
 		}
 		return reader, err
 	}
-	if err := m.managesChunksFile(chunkNum); err != nil {
+	if err := m.evictChunksFile(chunkNum); err != nil {
+		log.Println(err)
 		return reader, nil
 	}
 	return m.readChunk(reader, chunkNum)
@@ -115,24 +123,39 @@ func (m *Document) searchRead(reader *bufio.Reader, chunkNum int, searcher Searc
 
 // loadRead loads the read contents into chunks.
 func (m *Document) loadRead(reader *bufio.Reader, chunkNum int) (*bufio.Reader, error) {
-	m.currentChunk = chunkNum
 	if m.seekable {
-		if err := m.managesChunksFile(chunkNum); err != nil {
-			return reader, nil
-		}
-		if chunkNum != 0 {
-			m.loadedChunks.Add(chunkNum, struct{}{})
-		}
-		return m.readChunk(reader, chunkNum)
+		m.currentChunk = chunkNum
+		return m.loadReadFile(reader, chunkNum)
+	}
+	return m.loadReadMem(reader, chunkNum)
+}
+
+// loadReadFile loads the read contents into chunks.
+// loadReadFile frees old chunks and loads new chunks.
+func (m *Document) loadReadFile(reader *bufio.Reader, chunkNum int) (*bufio.Reader, error) {
+	_ = m.evictChunksFile(chunkNum)
+	m.addChunksFile(chunkNum)
+	if len(m.chunks[chunkNum].lines) != 0 {
+		// already loaded.
+		return reader, nil
+	}
+	return m.readChunk(reader, chunkNum)
+}
+
+// loadReadMem loads the read contents into chunks.
+// loadReadMem frees the memory behind and reads forward.
+func (m *Document) loadReadMem(reader *bufio.Reader, chunkNum int) (*bufio.Reader, error) {
+	if m.BufEOF() {
+		return reader, nil
 	}
 
-	if !m.BufEOF() {
-		if chunkNum < m.lastChunkNum() {
-			return reader, fmt.Errorf("%w %d", ErrAlreadyLoaded, chunkNum)
-		}
-		m.managesChunksMem(chunkNum)
-		m.requestContinue()
+	if chunkNum < m.lastChunkNum() {
+		// already loaded.
+		// return reader, fmt.Errorf("%w %d", ErrAlreadyLoaded, chunkNum)
+		return reader, nil
 	}
+	m.evictChunksMem(chunkNum)
+	m.requestContinue()
 	return reader, nil
 }
 
@@ -310,28 +333,19 @@ func (m *Document) fillChunk(chunk *chunk, reader *bufio.Reader, start int, isCo
 		atomic.StoreInt32(&m.changed, 1)
 		if err != nil {
 			if line.Len() != 0 {
-				if isCount {
-					m.append(chunk, line.Bytes())
-					m.offset = m.size
-				} else {
-					m.appendOnly(chunk, line.Bytes())
-				}
+				m.append(chunk, isCount, line.Bytes())
+				m.offset = m.size
+				atomic.StoreInt32(&m.noNewlineEOF, 1)
 			}
 			return err
 		}
-		if isCount {
-			m.append(chunk, line.Bytes())
-			m.offset = m.size
-		} else {
-			m.appendOnly(chunk, line.Bytes())
-		}
+		m.append(chunk, isCount, line.Bytes())
+		m.offset = m.size
 		line.Reset()
 	}
 	return nil
 }
 
-const bufSize = 4096
-
 // reserveChunk reserves ChunkSize lines.
 // read and update size only.
 func (m *Document) reserveChunk(reader *bufio.Reader, start int) error {
@@ -354,7 +368,7 @@ func (m *Document) countLines(reader *bufio.Reader, start int) (int, int, error)
 	for {
 		bufLen, err := reader.Read(buf)
 		if err != nil {
-			return count, size, err
+			return count, size, fmt.Errorf("read: %w", err)
 		}
 		if bufLen == 0 {
 			return count, size, io.EOF
@@ -378,6 +392,14 @@ func (m *Document) countLines(reader *bufio.Reader, start int) (int, int, error)
 		if num >= ChunkSize {
 			break
 		}
+		// no newline at the end of the file.
+		if bufLen < bufSize {
+			p := bytes.LastIndex(buf[:bufLen], []byte("\n"))
+			if p+1 < bufLen {
+				count++
+				atomic.StoreInt32(&m.noNewlineEOF, 1)
+			}
+		}
 	}
 	return count, size, nil
 }
@@ -393,9 +415,47 @@ func (m *Document) appendOnly(chunk *chunk, line []byte) {
 	m.mu.Unlock()
 }
 
-// append appends to the line of the chunk.
-// append updates the number of lines and size.
-func (m *Document) append(chunk *chunk, line []byte) {
+// joinLast joins the new content to the last line.
+// This is used when the last line is added without a newline and EOF.
+func (m *Document) joinLast(chunk *chunk, line []byte) bool {
+	if len(chunk.lines) == 0 {
+		return false
+	}
+	size := len(line)
+
+	m.mu.Lock()
+	num := len(chunk.lines) - 1
+	buf := chunk.lines[num]
+	dst := make([]byte, 0, len(buf)+size)
+	dst = append(dst, buf...)
+	dst = append(dst, line...)
+	m.size += int64(size)
+	chunk.lines[num] = dst
+	m.mu.Unlock()
+
+	m.cache.Remove(m.endNum - 1)
+	return true
+}
+
+// append appends a line to the chunk.
+func (m *Document) append(chunk *chunk, isCount bool, line []byte) {
+	if atomic.SwapInt32(&m.noNewlineEOF, 0) == 1 {
+		if m.joinLast(chunk, line) {
+			return
+		}
+	}
+
+	if isCount {
+		m.appendLine(chunk, line)
+		return
+	}
+
+	m.appendOnly(chunk, line)
+}
+
+// appendLine appends to the line of the chunk.
+// appendLine updates the number of lines and size.
+func (m *Document) appendLine(chunk *chunk, line []byte) {
 	m.mu.Lock()
 	size := len(line)
 	m.size += int64(size)
@@ -419,29 +479,10 @@ func (m *Document) appendFormFeed(chunk *chunk) {
 		if m.formfeedTime {
 			feed = fmt.Sprintf("%sTime: %s", FormFeed, time.Now().Format(time.RFC3339))
 		}
-		m.append(chunk, []byte(feed))
+		m.appendLine(chunk, []byte(feed))
 	}
 }
 
-// chunkForAdd is a helper function to get the chunk to add.
-func (m *Document) chunkForAdd() *chunk {
-	m.mu.Lock()
-	defer m.mu.Unlock()
-	if m.endNum < len(m.chunks)*ChunkSize {
-		return m.chunks[len(m.chunks)-1]
-	}
-	chunk := NewChunk(m.size)
-	m.chunks = append(m.chunks, chunk)
-	return chunk
-}
-
-// lastChunkNum returns the last chunk number.
-func (m *Document) lastChunkNum() int {
-	m.mu.Lock()
-	defer m.mu.Unlock()
-	return len(m.chunks) - 1
-}
-
 // reload will read again.
 // Regular files are reopened and reread increase.
 // The pipe will reset what it has read.
